#!/usr/local/bin/ruby
#
#----------------------------------------------------------------------
# 
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 2 of the License, or (at your option)
# any later version. This program is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
# Public License for more details; it is available at
# <http://www.fsf.org/copyleft/gpl.html>, or by writing to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
# 
# Written by Gordon Miller <gmiller@bittwiddlers.com>. Inspired by and partly
# derived from the Python version by Michael Haggerty. If you find a problem
# or have a suggestion, please let me know at <gmiller@bittwiddlers.com>.
# Other feedback would also be appreciated.
#
#----------------------------------------------------------------------
#
# This file contains the implementations of the Plot object (PlotBase, Plot,
# Splot, and Contour). This file should not be included directly by an
# application, the Gnuplot.rb file should be included instead.
#
#----------------------------------------------------------------------

class ContourParams
  attr_reader :npoints, :interpStyle, :nlevels

  def initialize ()
    @interpStyle = nil
    @npoints = 0
    @order = 1
    @nlevels = 0
    @auto = 0
    @discrete = nil
    @increments = nil
  end

  def getGnuplotCommand ()
    cmd = "cntrparam"

    if ( @interpStyle != nil ) then cmd = cmd + " #{@interpStyle}" end
    
    # If the interpolation style is bspline, then go ahead and put in the
    # points and order arguments
    #
    if ( @interpStyle == "bspline" )
      if ( @npoints > 0 ) then cmd = cmd + " points #{@npoints}"  end
      if ( @order > 0 ) then cmd = cmd + " order #{@order}"  end
    end
    
    # Now insert the level information.  There are a heck of a lot of
    # options here that are all mutually exclusive.
    #
    if ( @nlevels > 0 ) then
      cmd = cmd + " levels #{@nlevels}"

    elsif ( @auto > 0 ) then
      cmd = cmd + " levels auto #{@auto}"

    elsif ( @discrete != nil ) then
      cmd = cmd + " levels discrete " + @discrete.join(",")

    elsif ( @increments != nil ) then
      cmd = cmd + " levels incremental " + @increments.join(",")

    end
    
    return cmd
  end

  def setIncrements (start, incr, finish=nil)
    if ( start != nil and incr != nil )
      @increments = [start, inc, finish]
    else
      @increments = nil
    end
  end

  def setBspline (npoints = 2)
    @interpStyle = "bspline"
    @npoints = npoints
  end

  def setCubicSpline (npoints = 2)
    @interpStyle = "cubicspline"
    @npoints = 2
  end
  
  def setLinear (npoints = 1)
    @interpStyle = "linear"
    @npoints = 1
  end
  
  def setNlevels (nlevels)
    @nlevels = nlevels
  end
end

class ContourPlot < PlotBase
  
  def initialize (filename=nil, persist=0)
    super('splot', filename, persist)
    @rowSize = nil
    @colSize = nil
    @norm = nil
    @contourParams = nil
  end

  public
  def setGridMapping (rowSize=nil, colSize=nil, norm=nil)
    @rowSize = rowSize
    @colSize = colSize
    @norm = norm
  end

  public 
  def setContourParams (params)
    @contourParams = params
  end
  
  private
  def configurePlot
    super
    setString("view", "0, 0")
    setString("nosurface")
    setString("contour", "base")

    # Configure the Non-grid mapping.
    cmd = "dgrid3d"
    if ( @rowSize == nil and @colSize == nil and @norm == nil ) 
      cmd = "nodgrid3d"
    else
      if ( @rowSize != nil ) then cmd = cmd + " #{@rowSize}" end
      if ( @colSize != nil ) then cmd = cmd + ", #{@colSize}" end
      if ( @norm    != nil ) then cmd = cmd + ", #{@norm}" end
    end

    setString( cmd )

    if ( @contourParams )
      setString( @contourParams.getGnuplotCommand )
    end
  end
end
