#!/usr/local/bin/ruby
#
#----------------------------------------------------------------------
# 
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 2 of the License, or (at your option)
# any later version. This program is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
# Public License for more details; it is available at
# <http://www.fsf.org/copyleft/gpl.html>, or by writing to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
# 
# Written by Gordon Miller <gmiller@bittwiddlers.com>. Inspired by and partly
# derived from the Python version by Michael Haggerty. If you find a problem
# or have a suggestion, please let me know at <gmiller@bittwiddlers.com>.
# Other feedback would also be appreciated.
#
#----------------------------------------------------------------------
#
# This file contains the implementations of the Plot object (PlotBase, Plot,
# Splot, and Contour). This file should not be included directly by an
# application, the Gnuplot.rb file should be included instead.
#
#----------------------------------------------------------------------

# Base class for all objects that execute plotting within Gnuplot.  Each plot
# within Gnuplot is created using either a plot or splot command.  This
# basecommand is then passed arguments that specify what to plot and how to
# plot it.
#
# This class should not be directly instantiated but instead should be
# subclassed.
#
class PlotBase
  
  # The basic plotting command that will create this plot.
  #
  attr_reader :plotcmd
  
  # The set of PlotItem objects that are going to be plotted. 
  #
  attr_reader :itemList
  
  # Boolean flag controlling the output of debugging information.
  #
  attr_accessor :debug

  # Initialize a new object that will use the given plot command to execute
  # the specified gnuplot interpreter.  
  #
  def initialize (plotcmd, filename=nil, persist=1)
    @debug = nil
    @gnuplot = if filename == nil then
		 GnuplotProcessWriter.new(persist=persist)
	       else
		 GnuplotFileWriter.new(filename)
	       end

    @itemList = Array.new()
    @plotcmd = plotcmd
  end

  # Clear the current plot and create a new 2-d plot containing the specified
  # items. Each arguments should be of the following types:
  # 
  # 'PlotItem' (e.g., 'Data', 'File', 'Func') -- This is the most flexible way
  # to call plot because the PlotItems can contain suboptions. Moreover,
  # PlotItems can be saved to variables so that their lifetime is longer than
  # one plot command; thus they can be replotted with minimal overhead.
  # 
  # 'string' (e.g., 'sin(x)') -- The string is interpreted as 'Func(string)'
  # (a function that is computed by gnuplot).
  # 
  # Anything else -- The object, which should be convertible to an array, is
  # converted to a 'Data' item, and thus plotted as data. If the conversion
  # fails, an exception is raised.
  #
  def draw (*items)
    clearQueue()
    addToQueue(items)
    refresh()
  end

  # Send the string s as a command to gnuplot, followed by a
  # newline.  All communication with the gnuplot process (except
  # for inline data) is through this method.
  #
  def writeln(s)
    @gnuplot.writeln(s)
    puts "gnuplot #{s}" if @debug
  end
  
  # Method that is executed at the start of the plot to initialize the plot.
  # Typical use of this method is to execute 'set' statements for the plot.
  #
  def configurePlot
  end

  # Refresh the current plot by reissuing the gnuplot plot command
  # corresponding to the current itemList.
  #
  def refresh
    if ( @itemList.length > 0 ) 
      configurePlot()

      plotcmds = Array.new()

      @itemList.each { |item| plotcmds.push( item.command() ) }
      fullcmd = @plotcmd + " " + plotcmds.join(", ")
      writeln( fullcmd )
      
      @itemList.each { |item| item.pipein( @gnuplot ) }
      @gnuplot.flush()
    else
      puts "No items to plot"
    end
  end

  # Removes the items that currently reside in the plot item list.
  #
  def clearQueue
    @itemList.clear()
  end
  
  # Add a list of items to the itemlist (but don't plot them). 'items' is a
  # sequence of items, each of which should be a 'PlotItem' of some kind, a
  # string (interpreted as a function string for gnuplot to evaluate), or a
  # Numeric array (or something that can be converted to a Numeric array).
  #
  def addToQueue (items)
    if ( items.is_a? PlotItem ) then
      @itemList.push( items )

    elsif ( items.is_a? Array ) then
      items.each { |item|
	if ( item.is_a? PlotItem ) then
	  @itemList.push( item )
	end
      }
    end

  end

  # Replot the existing graph, using the items in the current itemlist. If
  # arguments are specified, they are interpreted as additional items to be
  # plotted alongside the existing items on the same graph. See 'plot' for
  # details.
  #
  def redraw (*items)
    addToQueue(items)
    refresh()
  end

  # Clear the plot window (without affecting the current itemlist).
  #
  def clear
    writeln("clear")
  end

  # Reset all gnuplot settings to their defaults and clear itemlist.
  #
  def reset
    writeln('reset')
    clearQueue()
  end

  # Insert a label with the given text into the plot at the specified position
  # and with the given justification.  If the position or justification are
  # not specified then they default to the gnuplot defaults.  Executing this
  # command is exactly equivalent to executing the gnuplot command.
  #
  # 'set #{text} [at #{position}] [#{justification}]'
  #
  def label (text, position = nil, justification = nil)
    args = "'#{text}'"

    if ( position )
      args += " at #{position}"
    end
    
    if ( justification )
      args += " #{justification}"
    end
    
    setString("label", args)
  end

  # Load a file using gnuplot's 'load' command.
  #
  def load(filename)
    writeln("load '#{filename}'")
  end

  # Invoked when there is no defined method for the given id.  The method call
  # is translated into a call to setString.
  #
  def method_missing (id, *args)
    name = id.id2name
    case args.length
    when 1
      setString( name, args.join )
    when 0
      setString(name)
    end
  end

  # Save the current plot commands using gnuplot's 'save' command.
  #
  def save(filename)
    writeln("save '#{filename}'")
  end

  # Set a string option, or if s is omitted, unset the option. If the method
  # determines that the option requires a quoted string, it will double
  # quote the string in the set call. Double quoting is performed so that
  # gnuplot will interpret backslash (\) characters. In order to prevent
  # ruby from interpreting the \ in characters like \n or \r, be sure to
  # escape the initial \. Therefore to set a linefeed, pass \\n or \\r
  # instead.
  #
  def setString(option, s=nil)
    requireString = ["xlabel", "output", "title", "ylabel"]

    if s == nil or s.length == 0 then
      writeln("set #{option}")
    else
      val = if requireString.include? option
	      "\"#{s}\""
	    else
	      s
	    end
      
      writeln("set #{option} #{val}")
    end
  end

  # Set the xrange of the plot.
  #
  def xrange (min, max)
    setString("xrange", "[#{min}:#{max}]")
  end

  # Set the yrange of the plot
  #
  def yrange (min, max)
    setString("yrange", "[#{min}:#{max}]")
  end

end

# Base class for all objects that create plots on a two-dimensional coordinate
# system.
#
class Plot < PlotBase
 
  # Initialize the plot.  If filename is specified then the result of the plot
  # will be a file containing the gnuplot commands.
  #
  def initialize (filename=nil,persist=1)
    super('plot', filename, persist)
  end

end

# Base class for all objects that create plots on a three-dimensional
# coordinate system.
#
class Splot < PlotBase
  
  # Initialize the plot.  If filename is specified then the result of the plot
  # will be a file containing the gnuplot commands.
  #
  def initialize (filename=nil,persist=1)
    super('splot', filename, persist)
  end

  # Set the zlabel for the plot.  If the label is not specified then the label
  # will be reset.
  #
  def setZlabel (s=nil)
    setString("zlabel", "'#{s}'")
  end
end

# $Id: Plot.rb,v 1.1.1.1 2002/06/04 12:36:28 gmiller Exp $
