#!/usr/local/bin/ruby
# 
#----------------------------------------------------------------------
# 
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 2 of the License, or (at your option)
# any later version. This program is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
# Public License for more details; it is available at
# <http://www.fsf.org/copyleft/gpl.html>, or by writing to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
# 
# Written by Gordon Miller <gmiller@bittwiddlers.com>. Inspired by and partly
# derived from the Python version by Michael Haggerty. If you find a problem
# or have a suggestion, please let me know at <gmiller@bittwiddlers.com>.
# Other feedback would also be appreciated.
#
#----------------------------------------------------------------------

# Base class of all objects that are capable of being plotted within Ruby
# Gnuplot. Child classes are categorized by the source of their data.
#
# == Options
#
# This base class understands and supports the following plotting options,
# which are common to all plot types.
#
# xrange - The range of values for the x axis
#
# yrange - The range of values for the y axis
#
# using  - The columns of a data that are to be used for the plot
# 
# with   - How the item should be plotted, e.g., with='points 3 3'.
#
# title  - The text to be associated with the item in the plot legend. If
#          this is set to 'nil', then this will cause the 'notitle' option to
#          be passed to gnuplot, which will cause the plot item to be omitted
#          from the legend.

class PlotItem

  # The first option that will be passed to the 'plot' or 'splot' command in
  # gnuplot. This is a characteristic of the derived class. The basecommand is
  # the string that tells gnuplot what its plotting. For example, in the
  # command 'plot sin(x)' the basecommand is 'sin(x)'. Probably not a very
  # intuitive name, but it works for now.
  #
  attr_reader :basecommand

  # Returns the set of options that will be used in the call to the base
  # command. 
  #
  attr_reader :options

  # The order in which options need to be passed to gnuplot.
  #
  @@optionSequence = ['xrange', 'yrange', 'binary', 'using', 'title', 'with']
  
  # Construct a new PlotItem object that has the specified basecommand and
  # options.
  #
  def initialize (basecommand, opts=nil)
    @basecommand = basecommand
    @options = Hash.new

    if opts
      opts.each { |n,v| @options[n] = v }
    end
  end

  # Return the setting of an option.
  def getOption(name)
    return @options[name]
  end

  # Clear (unset) a plot option.  No error if option was not set.
  def clearOption(name)
    @options.delete(name)
  end

  # Build and return the 'plot' command, with options, necessary
  # to display this item.
  # 
  def command

    cmd = ""

    if ( @options["xrange"] != nil )
      cmd += " " + @options["xrange"]
    end
    
    if ( @options["yrange"] != nil and @options["xrange"] != nil )
      cmd += " " + @options["yrange"]
    end

    cmd += " " + @basecommand

    if ( @options["using"] != nil )
      cmd += sprintf(" using %s", @options["using"] )
    end

    titleStr = @options["title"]
    cmd += case titleStr
	   when "notitle" then "notitle"
	   when nil then ""
	   else sprintf(" title '%s'", titleStr)
    end

    if ( @options["with"] != nil ) 
      cmd += sprintf(" with %s", @options["with"])
    end

    return cmd
  end

  # Invoked by Ruby when a method is invoked on this class that doesn't
  # exist.  The method name is taken to be the name of an option and the
  # argument is taken to be a value.  The name/value pair is inserted into 
  # the options hash.
  #
  def method_missing (id, *args)

    name = id.id2name
    case args.length
    when 1
      @options[name] = args[0]
    when 0
      return @options[name]
    end
  end

  # Pipe necessary inline data to gnuplot.
  # 
  # If the plot command requires data to be put on stdin (i.e.,
  # 'plot "-"'), this method should put that data there.  Can be
  # overridden in derived classes.
  # 
  def pipein(f)
  end
end


# A PlotItem that understands how to plot instances of Array objects. The
# basecommand that is used in the plot and splot commands is the '-' character
# which tells Gnuplot that the data is inline. The pipein method is then used
# after the plot command to write out the data into the GnuplotWriter that is
# the sink for the Gnuplot commands.
#
# == Options
# 
# xgrid - The x grid for the data object.  Must be of the same length as the
#         first dimension of the data array.
# 
class ArrayData < PlotItem

  # Constructs a new object that will plot the specified data. The data
  # parameter is expected to be an instance of an Array object. The parameters
  # that are understood by this object will used and removed and the remaining
  # options will be passed to the PlotItem initialize method. That means that
  # the opts object is modified while in this method.
  #
  def initialize (data, opts=nil)
    super("'-'", opts)
    @data = data
  end

  # Overrides the PlotItem.pipein method and simply calls the writeData method
  # with the writer.
  #
  def pipein(f)
    writeData(f)
    f.write("e\n")
  end

  # Writes the data member of the object into the specified GnuplotWriter w. 
  # The data is written in the Gnuplot format which is the one data point per
  # line.
  # 
  # If the data member is one dimensional, then the data inserted into the
  # writer will have two columns. The first column contains the x value of the
  # point and the second will contain the y value. If the data is two
  # dimensional, then the output will contain three columns, x, y, and z.
  # 
  def writeData (f)
    xgrid = getOption("xgrid")
    sigma = getOption("sigma")

    @data.length.times { |i|
      x = if xgrid then xgrid[i].to_s else i.to_s end
      sig = if sigma then sigma[i].to_s else " " end

      f.writeln( x + " " + @data[i].to_s + " " + sig)
    }
  end
end
  
# StringFunc represents a mathematical expression that is to be computed by
# gnuplot itself.  For example, creating a StringFunc object with the string
# 'sin(x)' like so:
#
# StringFunc('sin(x)')
# 
# will have the same effect as typing the following into gnuplot itself:
# 
# 'plot sin(x)'
#
class StringFunc < PlotItem

  # Initializes a new StringFunc object that will have gnuplot plot the
  # specified function. The str argument is expected to be a string that
  # represents a gnuplot function. The opts are passed directly to the base
  # class.
  # 
  def initialize (str, opts = {})
    super
  end
end


# $Id: PlotItem.rb,v 1.1.1.1 2002/06/04 12:36:28 gmiller Exp $
