#!/usr/local/bin/ruby
#
#---------------------------------------------------------------------- 
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 2 of the License, or (at your option)
# any later version. This program is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
# Public License for more details; it is available at
# <http://www.fsf.org/copyleft/gpl.html>, or by writing to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
# 
# Written by Gordon Miller <gmiller@bittwiddlers.com>. Inspired by and partly
# derived from the Python version by Michael Haggerty. If you find a problem
# or have a suggestion, please let me know at <gmiller@bittwiddlers.com>.
# Other feedback would also be appreciated.
#
#----------------------------------------------------------------------

# Base class for all objects that can receive Gnuplot commands. An instance of
# this class acts like a stream into which the gnuplot commands are inserted. 
#
# Child class of GnuplotWriter must implement the write and flush methods.
#
class GnuplotWriter
  
  # Writes the given string to the object using the write method and invokes
  # flush afterwards.
  #
  def writeln (s)
    write(s + "\n")
    flush()
  end

end

# This class keeps a string of commands that are written to the GnuplotSink.
# Each call to a command that passes data into the class writes the data to a
# member string that can then be accessed via the data accessor. The principle
# purpose of this class is for use in testing.
# 
class GnuplotStringWriter < GnuplotWriter
  
  # The string that is the accumulator for the data that is written to this
  # object.
  #
  attr_reader :data

  # Initializes the instance to contain an empty data member.
  #
  def initialize ()
    @data = ""
  end

  # Appends the contents of the string s onto the data member.
  #
  def write (s)
    @data += s
  end
  
end

# Sometimes it is convenient to write gnuplot commands to a command file for
# later evaluation. In that case, one of these objects is used as a mock
# gnuplot process. Note that temporary files may be deleted before you have
# time to execute the file!

# 
class GnuplotFileWriter < GnuplotWriter

  # Initialize the instance to write to the specified filename
  #
  def initialize (filename)
    @file = open(filename, 'w')
    writeln("#!/bin/env gnuplot")
  end

  # Write an arbitrary string to the file.
  #
  def write (s)
    @file.write(s)
  end
  
  # Cause pending output to be written immediately.
  #
  def flush
    @file.flush
  end
end

# This represents a running gnuplot program and the means to communicate with
# it at a primitive level (i.e., pass it commands or data). When the object is
# destroyed, the gnuplot program exits (unless the 'persist' option was set). 
# The communication is one-way; gnuplot's text output just goes to stdout with
# no attempt to check it for error messages.
#
class GnuplotProcessWriter < GnuplotWriter

  # The command required to start the gnuplot interpreter.
  #
  attr_reader :cmd

  # Create a 'GnuplotProcess' object. This starts a gnuplot program and
  # prepares to write commands to it.
  #
  # persist - If set to true then the gnuplot command will be run with the
  #           'persist' command line option. 
  # 
  def initialize (persist = true)

    # Try to determine the path to the gnuplot command.  This should work
    # under csh on unix but I have no idea how to do it under Windows.
    #
    IO.popen ( "which gnuplot" ) { |io| @cmd = io.readline.chomp }
    @cmd += " -persist" if persist

    @gnuplot = IO::popen(@cmd, "w")
  end

  # Write pending output immediately
  #
  def flush
    @gnuplot.flush
  end

  # Pass an arbitrary string to the gnuplot program
  #
  def write (s)
    @gnuplot.write( s )
  end
  
end

# $Id: Writer.rb,v 1.1.1.1 2002/06/04 12:36:28 gmiller Exp $
